@@ -88,6 +88,7 @@ def refine_r(
     d_low,
     r_max,
     custom_cost,
+    verbose=False,
     ):
 
     ub = np.inf*np.ones(r0.size)
@@ -101,7 +102,6 @@ def refine_r(
             A[l,l+1] = -1
     constr = LinearConstraint(A, lb=np.ones(r0.size), ub=np.inf*np.ones(r0.size))
     
-    verbose = 0
     try:
         result = minimize(
             cg.cost,
@@ -118,7 +118,7 @@ def refine_r(
             constraints=constr,
             bounds=bnd,
             options={
-                'verbose': verbose,
+                'verbose': 2 if verbose and comm.rank==0 else 0,
                 'gtol': 1e-8,
                 'xtol': 1e-8,
                 'maxiter': 1000,
@@ -226,6 +226,7 @@ def insert_needle(
     d_low,
     custom_cost,
     lmax=None,
+    verbose=False,
     ):
     """ mat_dict: database of all materials in the multilayer stack
         mat_needle: list of materials that can be inserted as needles (array of strings)
@@ -363,6 +364,22 @@ def insert_needle(
             intervals = np.size(loc_temp)
             loc_copy = loc_temp.copy()
             cnt = 0
+            if verbose and comm.rank==0:
+                print('Minima Locations Identified    : ', end='', flush=True)
+                for ind in range(intervals):
+                    print(str(np.round(loc_copy[ind], 2)) + ' | ', end='', flush=True)
+                print('', flush=True)
+                
+                print('Topology Derivatives at Minima : ', end='', flush=True)
+                for ind in range(dMF_temp.size):
+                    print(str(np.round(dMF_temp[ind], 2)) + ' | ', end='', flush=True)
+                print('', flush=True)
+                
+                print('Intervals Being Searched       :   ', end='', flush=True)
+                for ind in range(evaluate.size):
+                    print(str(evaluate[ind]) + ' | ', end='', flush=True)
+                print('', flush=True)
+                
             for i in range(intervals-2, -1, -1):
                 if evaluate[i]:
                     result = minimize_scalar(
@@ -401,6 +418,7 @@ def insert_needle(
                         ),
                         bounds=loc_copy[i:i+2],
                         method='bounded',
+                        options={'disp': 3 if verbose and comm.rank==0 else 0},
                     )
                                              
                     nfev += result.nfev
@@ -446,6 +464,7 @@ def deep_search(
     r_max,
     custom_cost,
     lmax=None,
+    verbose=False,
     ):
                 
     MF_deep = np.zeros(mat_needle.size)
@@ -497,6 +516,7 @@ def deep_search(
                     d_low,
                     r_max,
                     custom_cost,
+                    verbose=verbose,
                 )
                 
             r_out[m,z] = r_new
@@ -620,6 +640,7 @@ def run_needle(
     max_layers,
     custom_cost,
     lmax=None,
+    verbose=False,
     ):
                
     ml_init = multilayer(lam_cost, theta_cost, phi_cost)
@@ -636,8 +657,15 @@ def run_needle(
             d_low,
             r_max,
             custom_cost,
+            verbose=verbose,
         )
     
+    if verbose and comm.rank == 0:
+        print('Current Design: ', end='', flush=True)
+        for ind in range(r_new.size):
+            print(str(np.round(r_new[ind], 1)) + ' | ', end='', flush=True)
+        print('', flush=True)
+    
     mat_profile_new = mat_profile.copy()
     n_new = n.copy()
     ban_needle_new = ban_needle.copy()
@@ -660,8 +688,15 @@ def run_needle(
             d_low,
             custom_cost,
             lmax=lmax,
+            verbose=verbose,
         )
         
+        if verbose and comm.rank == 0:
+            if needle_status:
+                print('Needle Insertion Location Found', flush=True)
+            else:
+                print('Needle Insertion Not Possible', flush=True)
+        
         if needle_status == 0:
             break
         
@@ -681,8 +716,15 @@ def run_needle(
                 r_max,
                 custom_cost,
                 lmax=lmax,
+                verbose=verbose,
             )
         
+        if verbose and comm.rank == 0:
+            print('Current Design: ', end='', flush=True)
+            for ind in range(r_new.size):
+                print(str(np.round(r_new[ind], 1)) + ' | ', end='', flush=True)
+            print('', flush=True)
+        
         thickness = r_new[:-1] - r_new[1:]
         if np.sum(thickness < d_low) > 1:
             break
@@ -736,6 +778,7 @@ def run_needle(
                 d_low,
                 r_max,
                 custom_cost,
+                verbose=verbose,
             )
             
         n_new = n_fin.copy()
@@ -775,6 +818,7 @@ def radius_sweep(
     custom_cost,
     mat_data_dir=None,
     lmax=None,
+    verbose=False,
     ):
 
     # Create n
@@ -837,6 +881,7 @@ def radius_sweep(
             max_layers,
             custom_cost,
             lmax=lmax,
+            verbose=verbose,
         )
         
         assert r_init.size == 1
@@ -908,6 +953,7 @@ def radius_sweep(
                 max_layers,
                 custom_cost,
                 lmax=lmax,
+                verbose=verbose,
             )
         
         radius_proc[nr] = r_fin
